/*****
 *
 * ACHOICE2.PRG
 *
 * Luiz Quintela
 * Copyright (c) 1990 Nantucket Corporation.
 *                    All Rights Reserved.
 *
 */

// Header Files
#include "inkey.ch"
#include "achoice2.ch"    // Your own header
#include "setcurs.ch"

// Global
STATIC aStru
STATIC n

/*****
 *
 * ACHOICE2()
 *
 */

FUNCTION Achoice2( nTop, nLeft, nBottom, nRight, aItems,;
                   aLogic, aMsg, nMsgLn, bBlock )

// Achoice2() receives the following parameters:

// nTop, nLeft, nBottom, nRight  -  window coordinates
// aItems  -  Array with Menu Items
// aLogic  -  Array with .T. or .F. for each item in menu
// aMsg    -  Array with messages for each item in menu
// nMsgLn  -  Line to display messages
// bBlock  - Codeblock to be executed while waiting for a key

LOCAL b, nKey, column
LOCAL x, k
LOCAL nNewPos
LOCAL cSaveClr, lIsMsg
LOCAL nCursSave := SETCURSOR(SC_NONE)

aStru := ARRAY( ARRAY_SIZE )

// This Array is Pure Knowledge
aStru.top     := nTop
aStru.left    := nLeft
aStru.bottom  := nBottom
aStru.right   := nRight

aStru.items   := aItems
aStru.alen    := LEN( aItems )

IF VALTYPE(aMsg) == "A"
   aStru.msg := aMsg
   lIsMsg    := TRUE

ELSE
   lIsMsg := FALSE

ENDIF
aStru.msglen := IF(nMsgLn == NIL, aStru.bottom + 5, nMsgLn)

IF VALTYPE(aLogic) != "A"
   aStru.logic := ARRAY( aStru.alen )
   AFILL( aStru.logic, TRUE )

ELSE
   aStru.logic := aLogic

ENDIF

aStru.initpos := 0
aStru.window  := aStru.bottom - aStru.top
aStru.width   := aStru.right - aStru.left - 1

// To avoid internal errors
bBlock := IF(bBlock == NIL, {|| .F.}, bBlock)

// Draw the Box and Gauge
cSaveClr := SETCOLOR("BG/B")
@ aStru.top - 1, aStru.left - 1 CLEAR TO ;
  aStru.bottom + 1, aStru.right + 1
@ aStru.top - 1, aStru.left - 1 TO ;
  aStru.bottom + 1, aStru.right + 1
@ aStru.top,aStru.right + 1,;
  aStru.bottom,aStru.right + 1 BOX CHR(176) ;
                               COLOUR "GR+/N"
@ aStru.top,aStru.right + 1 SAY CHR(178) ;
                                COLOUR "R+/N"

// Create a Browse Object
b := TBrowseNew( aStru.top, aStru.left,;
                 aStru.bottom, aStru.right )
//
// TBrowseNew() creates a TBrowse object with no columns 
// and no code blocks for data source positioning
// You need to supply your own
// 
// n is the array subscript 
//

n := 1

//
// Go Top and Go Bottom Blocks
//
// Position: First Element
b:goTopBlock    := { || n := 1 }
//
// Position: Last Element
b:goBottomBlock := { || n := aStru.alen }
//
// This is our Skip Block
//
b:skipBlock := {|x| k := IF(ABS(x) >= IF(x >= 0,;
                         aStru.alen - n, n - 1),;
                         IF(x >= 0, aStru.alen - n, 1 - n),;
                         x), n += k, ;
                         k }

// Create the column object
column := TBColumnNew ( , {|x| aStru.items[n] } )
// We are going to use some colors for this column
// You are going to use the instance variable colorSpec,
// which contains a character string defining a color 
// table to the browse display.
// As a default, the current SETCOLOR is copied into this 
// variable
b:colorSpec := "BG/B,GR+/R,W/N,N,GR+/W"

//
// Colors for Selectable and Unselectable Items
//
column:colorBlock := {|x| IF( aStru.logic[n], {1, 2}, {1, 5})}
column:width      := aStru.width
//
// About Column Width:
//
// When browsing array elements, you need to pay attention
// to their size. This is very importante because TBcolumn
// will work based in the size of the first element of
// the array being browsed.
// We are assuming the width between left and right margins
//
// Add the column to the browse

b:addColumn( column )

WHILE TRUE
   // Stabilization
   WHILE !(b:stabilize()) .AND. ((nKey := INKEY()) == 0)
   END

   IF ( b:stable )
      IF ( b:hitTop .OR. b:hitBottom )
         TONE(87.3,1)
         TONE(40,3.5)

      ENDIF
      // Update Bar Gauge
      nNewPos := aStru.window / (aStru.alen / n)
      IF n == 1
         nNewPos := 0

      ELSEIF n == aStru.alen
         nNewPos := aStru.window

      ENDIF
      IF aStru.initpos != nNewPos
         @ aStru.top + aStru.initpos,;
           aStru.right + 1 SAY CHR(176) COLOUR "GR+/N"
         @ aStru.top + nNewPos, aStru.right + 1;
           SAY CHR(178) COLOUR "R+/N"
         aStru.initpos := nNewPos

      ENDIF
      // Messages?
      IF lIsMsg
         @ aStru.msglen, aStru.left - 1 SAY aStru.msg[n] ;
                                        COLOUR "GR+/N"

      ENDIF
      WHILE ((nKey := INKEY(0.1)) == 0)
         EVAL(bBlock)

      END

   ENDIF

   // Process key
   IF ( nKey == K_DOWN )
      b:down()

   ELSEIF ( nKey == K_UP )
      b:up()

   ELSEIF ( nKey == K_PGDN )
      b:pageDown()

   ELSEIF ( nKey == K_PGUP )
      b:pageUp()

   ELSEIF ( nKey == K_CTRL_PGUP ) .OR.;
          ( nKey == K_HOME )
      b:goTop()

   ELSEIF ( nKey == K_CTRL_PGDN ) .OR.;
          ( nKey == K_END )
      b:goBottom()

   ELSEIF ( nKey == K_ESC )
      SETCOLOR(cSaveClr)
      SETCURSOR(nCursSave)
      RETURN 0

   ELSEIF ( nKey == K_ENTER )
      // Is the item selectable?
      IF aStru.logic[n]
         // If so, return array subscript for 
         // the element
         SETCOLOR(cSaveClr)
         SETCURSOR(nCursSave)
         RETURN n

      ENDIF

   ENDIF

END
RETURN NIL

// EOF - ACHOICE2.PRG //
